package find;

/**
 * Calling: Browse(int mode, String getClass)
 * mode = 0-2 files_only, directories_only, files_and_directories
 * getClass is the name of caller's class this.getClass().toString() or simply the name of program
 * it is written under system default directory
 * f.ex. savePathFileName is "savePath_Find.txt" Find is a program name
 */
import java.awt.Container;
import java.io.File;
import javax.swing.JFileChooser;
import javax.swing.JOptionPane;

/**
 * @author Veikko Niemi 11.12.2008 9.21.10
 */
public class Browse {

    public static final int FILES_ONLY = 0;
    public static final int DIRECTORIES_ONLY = 1;
    public static final int FILES_AND_DIRECTORIES = 2;
    private int browseMode; // 0, 1, 2
    private static File[] inputs;
    private String saveFileName; // name of path file for next run
    private File savePathNamePath; // location of path for next run
    String s = System.getProperty("file.separator");
    private JFileChooser fileChooser;
    private File savePath;
    private String lS = System.getProperty("line.separator");

    public Browse() {
    }

    public Browse(int browseMode, String getClass) {
        /* browseMode = 0=files only, 1=catalogs only, 2= files and catalogs
         * getClass = name of calling class
         */
        if (browseMode > -1 && browseMode < 3) {
            this.browseMode = browseMode;
        } else {
            this.browseMode = JFileChooser.FILES_AND_DIRECTORIES;
        }
        String[] fns = getClass.split("[. ]");
        String fn = fns[fns.length - 1];
        this.saveFileName = "savePath_" + fn + ".txt";
    }

    public int getBrowseMode() {
        return browseMode;
    }

    public File[] getInputs() {
        return inputs;
    }

    public String getSaveFileName() {
        return saveFileName;
    }

    public void setBrowseMode(int browseMode) {
        this.browseMode = browseMode;
    }

    public void setSaveFileName(String saveFileName) {
        this.saveFileName = saveFileName;
    }

    public boolean browse() {
        fileChooser = new JFileChooser();
        setDefaultPath();
        fileChooser.setMultiSelectionEnabled(true);
        fileChooser.setFileSelectionMode(this.browseMode);
        int result = fileChooser.showOpenDialog(null);
        boolean ok = true;
        if (result == JFileChooser.ERROR) {
            ok = false;
        }
        if (result == JFileChooser.CANCEL_OPTION) {
            ok = false;
        }
        inputs = fileChooser.getSelectedFiles();
        File saveParent;
        if (inputs.length > 0) {
            saveParent = inputs[0].getParentFile();
            writeDefaultPath(saveParent);
        }
        return ok;
    }

    private void setDefaultPath() {
        savePathNamePath = fileChooser.getCurrentDirectory();
        File lT = new File(savePathNamePath + s + saveFileName);
        if (lT.exists()) {
            ReadFile lueT = new ReadFile(lT);
            String savePathFileS = lueT.readNext(); // read previously saved path
            lueT.close();
            savePath = new File(savePathFileS);
            fileChooser.setCurrentDirectory(savePath); // new default path
        }
    }

    private void writeDefaultPath(File saveParent) {

        fileChooser.setCurrentDirectory(saveParent);
        WriteFile kirT = new WriteFile(new File(savePathNamePath + s + saveFileName));
        kirT.writeLine(saveParent.toString());
        kirT.closeFile();
    }

    private void error(String show) {
        JOptionPane.showMessageDialog(null, show, "error",
                JOptionPane.ERROR_MESSAGE);
    }
}