/*
 * Luokkaa kutsutaan antamalla parametreiksi Browse luokan tuottaman taulukon
 * valituista hakemistoista/tiedostoista sekä halutuista tiedostotyypeistä taulukossa
 * tuloksena voidaan kutsua metodia getFiles joka palauttaa ArrayListin valituista fiedostoista
 */
package find;

import java.io.File;
import java.text.DecimalFormat;
import java.util.ArrayList;
import javax.swing.JOptionPane;
import javax.swing.JTextArea;

/**
 * Created on 23.12.2008, 5:50:02
 * @author Veikko Niemi
 */
public class WalkFiles {

    private File f;
    private ArrayList<File> cats;     // help array of folders to be searched
    private ArrayList<File> files;      // all filttered files
    private File[] inF;
    private String[] fTypes;        // F.ex "java, htm" or empty=all
    private int filecount;
    private int totalFilesCount;
    private int foldersCount;
    private int ifPrint = 0;            // 0 = no printing
    public static final int PRINTALLFILESNAMES = 1;
    public static final int PRINTALLSELECTEDFILESNAMES = 2;
    private JTextArea tarea;
    private String ls = System.getProperty("line.separator");
    private long selectedSpace;    // total space of all selected files in bytes
    protected static boolean printFirstStack = true;
    private int lockFilesCount;
    protected static double estimatedTime;

    public WalkFiles() {
    }

    public WalkFiles(File[] inF, String[] fTypes, JTextArea tarea) {
        this.inF = inF;
        this.fTypes = fTypes;
        this.tarea = tarea;
        printFirstStack = true;
        lockFilesCount = 0;
    }

    public WalkFiles(WalkFiles wf) {
        this.inF = wf.inF;
        this.fTypes = wf.fTypes;
        this.tarea = wf.tarea;
        printFirstStack = true;
        lockFilesCount = 0;
    }

    public String[] getFTypes() {
        return this.fTypes;
    }

    public int getTotalFilesCount() {
        return this.totalFilesCount;
    }

    public int getFoldersCount() {
        return this.foldersCount;
    }

    public ArrayList<File> getFiles() {
        walkFiles();
        return files;
    }

    public File[] getInF() {
        return inF;
    }

    public int getLockFilesCount(){
        return lockFilesCount;
    }
    public void setFTypes(String[] fTypes) {
        this.fTypes = fTypes;
    }

    public void setFiles(ArrayList<File> files) {
        this.files = files;
    }

    public void setInF(File[] inF) {
        this.inF = inF;
    }

    public void setIfPrint(int option) {
        if (option == PRINTALLFILESNAMES) {
            ifPrint = PRINTALLFILESNAMES;
        }
        if (option == PRINTALLSELECTEDFILESNAMES) {
            ifPrint = PRINTALLSELECTEDFILESNAMES;
        }
    }

    @Override
    public String toString() {
        String palauta = "";
        if (inF != null) {
            palauta = "Searching catalogs = ";
            for (int i = 0; i < inF.length; i++) {
                palauta += inF[i] + "\n";
            }
            if (fTypes != null) {
                palauta += "of filetypes = ";
                for (int i = 0; i < fTypes.length - 1; i++) {
                    palauta += fTypes[i] + ", ";
                }
                if (fTypes.length > 0) {
                    palauta += fTypes[fTypes.length - 1];
                }
            } else {
                palauta = "";
            }
        }
        return palauta;
    }

    private void walkFiles() {
        cats = new ArrayList<File>();   // table of folders to be searched
        files = new ArrayList<File>();  // filttered files
        tarea.append("Starting catalogs/files:" + ls);
        for (int i = 0; i < inF.length; i++) {
            cats.add(inF[i]);
            tarea.append(inF[i] + ls);
        }
        filecount = 0;
        makeAList();
    }

    private void estimateTime(long totalSpace) {
        DecimalFormat d = new DecimalFormat("###,###,###,###,##0");
        long defaultSpace = 35000; // 35kb
        int defaultTime = 4; // defaultSpace uses defaultTime in ms
        estimatedTime = totalSpace / defaultSpace * defaultTime / 1000;
        tarea.append("Total space " + d.format(totalSpace) + " bytes" + ", files " + filecount + ls);
        DecimalFormat df = new DecimalFormat("####0.0s");
//        Find.estimatedTime.setText("<html>Total space " + d.format(totalSpace) + " bytes, files " + filecount + "<br/>" +
//               "Estimated time " + df.format(estimatedTime));
        attention("Total space " + d.format(totalSpace) + " bytes" + ", files " + filecount + ls +
                "DefaultSpace " + defaultSpace + " bytes " + " DefaultTime " + defaultTime + " ms"+ ls +
              "Estimated time " + df.format(estimatedTime) + ls );
        tarea.append("DefaultSpace " + defaultSpace + " bytes " + " DefaultTime " + defaultTime + " ms" + ls);
        tarea.append("Estimated time " + df.format(estimatedTime) + ls);
        System.out.println("DefaultSpace " + defaultSpace + " bytes " + " DefaultTime " + defaultTime + " ms");
        System.out.println("Estimated time " + df.format(estimatedTime));
    }

    private long makeAList() {
        // returns total length of all files in bytes
        this.selectedSpace = 0;    // space of all files in bytes
        int ind = 0;
        foldersCount = 0;
        totalFilesCount = 0;
        String[] directory;
        String s = System.getProperty("file.separator");
        if (cats.size() > 0) {
            do {
                f = cats.get(ind);
                if (f.isDirectory()) {
                    foldersCount++;
                    directory = f.list();
                    ArrayList<File> help = new ArrayList<File>();
                    for (int i = 0; i < directory.length; i++) {
                        if (directory[i] != null) {
                            directory[i] = f + s + directory[i];
                            help.add(new File(directory[i]));
                        }
                    }
                    for (int i = 0; i < help.size(); i++) {
                        File h = help.get(i);
                        if (h.isDirectory()) {
                            cats.add(h);
                        } else {
                            if (h.isFile()) {
                                totalFilesCount++;
                                if (filtter(h)) {
                                    files.add(h);
                                    selectedSpace += h.length();
                                // System.out.println("File " + h + " size " + f.length() + " bytes");
                                }
                            }
                        }
                    }
                } else {
                    if (f.isFile()) {
                        totalFilesCount++;
                        if (filtter(f)) {
                            files.add(f);
                            selectedSpace += f.length();
                        // System.out.println("File " + h + " size " + f.length() + " bytes");
                        } else {
                        }
                    }
                }
                ind++;
            } while (ind < cats.size());
        }
        estimateTime(selectedSpace);
        return selectedSpace;
    }

    private boolean lockFiles(File f) {
        if (fTypes.length == 0 && f.toString().toLowerCase().endsWith(".lock")) {
            lockFilesCount++;
            return true;
        }
        return false;
    }

    private boolean filtter(File f) {
        boolean select = false;
        if (!lockFiles(f)) {
            String name = f.getName();

            if (fTypes.length > 0) {
                String[] exts = name.split("[.]");
                if (exts.length > 1) {
                    String ext = exts[exts.length - 1]; // last extension html f.ex
                    for (int i = 0; i < fTypes.length; i++) {
                        if (ext.equalsIgnoreCase(fTypes[i])) {
                            select = true;
                            filecount++;
                            if (ifPrint == PRINTALLSELECTEDFILESNAMES) {
                                System.out.println(filecount + ". " + f);
                            }
                            break;
                        }
                    }
                }
            } else {
                select = true;
                filecount++;
                if (ifPrint == PRINTALLSELECTEDFILESNAMES) {
                    System.out.println(filecount + ". " + f);
                }
            }
            if (ifPrint == PRINTALLFILESNAMES) {
                System.out.println(totalFilesCount + ". " + f);
            }
        }
        return select;
    }

    protected final void attention(final String show) {
        java.awt.EventQueue.invokeLater(new Runnable() {
            public void run() {
               JOptionPane.showMessageDialog(null, show, "attention",
                JOptionPane.INFORMATION_MESSAGE);
            }
        });
    }
}
